//+------------------------------------------------------------------+
//|                                               GoldenCrossBuy.mq5 |
//|                                                  Sudhanshu Gupta |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Sudhanshu Gupta"
#property link      "https://www.mql5.com"
#property version   "1.00"
#include<Trade\Trade.mqh>
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
// Define input parameters
input int shortTermPeriod_Buy = 33;  // Short-term EMA period
input int longTermPeriod_Buy = 41;  // Long-term EMA period
input int atrPeriod = 14;  // ATR period
input double atrMultiplier = 2.0;  // ATR multiplier for trailing stop

// Define global variables
double stoploss=0.0,takeprofit=0.0;
bool isOrderPlaced=false;
int ENUM_OPEN_POSITION_TYPE=0;
CTrade Trade;

int OnInit()
  {
//---
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
    MqlTick Latest_Price; // Structure to get the latest prices    
    SymbolInfoTick(Symbol() ,Latest_Price); // Assign current prices to structure 
    
    //create an array for several prices
    double shortTermMA_Buy[], longTermMA_Buy[], valueATR[];

    // Calculate short-term EMA
    double shortTermEMA_Buy = iMA(_Symbol, PERIOD_H1, shortTermPeriod_Buy, 0, MODE_EMA, PRICE_CLOSE);

    // Calculate long-term EMA
    double longTermEMA_Buy = iMA(_Symbol, PERIOD_H1, longTermPeriod_Buy, 0, MODE_EMA, PRICE_CLOSE);

    // Calculate current ATR
    double currentATR = iATR(_Symbol, PERIOD_H1, atrPeriod);
    
    //sort the price arrays 1, 2 from current candle
    ArraySetAsSeries(shortTermMA_Buy,true);
    ArraySetAsSeries(longTermMA_Buy,true);
    ArraySetAsSeries(valueATR,true);
    
    //Defined MA1, MA2 - one line - currentcandle, 3 candles - store result
    CopyBuffer((int)shortTermEMA_Buy,0,0,3,shortTermMA_Buy);
    CopyBuffer((int)longTermEMA_Buy,0,0,3,longTermMA_Buy);
    CopyBuffer((int)currentATR,0,0,3,valueATR);

    // Check for a buy crossover
    if (shortTermMA_Buy[0] > longTermMA_Buy[0] && shortTermMA_Buy[1] <= longTermMA_Buy[1] && !isOrderPlaced)
    {
        double volume=1.0;
        stoploss=Latest_Price.ask-atrMultiplier*valueATR[0];
        isOrderPlaced=SendOrder(ORDER_TYPE_BUY, volume, stoploss, takeprofit);
        ENUM_OPEN_POSITION_TYPE=1;
    }
    
    if(isOrderPlaced && isNewBar()){
      PositionSelect(_Symbol);
      Print("Total positions:",PositionsTotal());
      ulong resultOrder=PositionGetTicket(0);
      Print("Ticket:",resultOrder);
      
      if(ENUM_OPEN_POSITION_TYPE==1){
         stoploss=Latest_Price.ask-atrMultiplier*valueATR[0];
      }else if (ENUM_OPEN_POSITION_TYPE==-1){
         stoploss=Latest_Price.ask+atrMultiplier*valueATR[0];
      }else{
         stoploss=0;
      }
      
      takeprofit=0;
      modifyPosition(resultOrder,stoploss,takeprofit);
    }
  }

void OnTrade(){
   if(PositionsTotal()==0){
      isOrderPlaced=false;
   }
}

bool SendOrder(ENUM_ORDER_TYPE orderType, double vol,double sl,double tp){
   //--- prepare a request
   MqlTradeRequest request={};
   request.action=TRADE_ACTION_DEAL;         // setting a pending order
   request.magic=OrderGetInteger(ORDER_MAGIC);                  // ORDER_MAGIC
   request.symbol=_Symbol;                      // symbol
   request.volume=vol;                          // volume in 0.1 lots
   request.sl=sl;                                // Stop Loss is not specified
   request.tp=tp;                                // Take Profit is not specified   
   //--- form the order type
   request.type=orderType;                // order type
//--- form the price for the pending order
   if (orderType==ORDER_TYPE_BUY){
      request.price=SymbolInfoDouble(Symbol(),SYMBOL_ASK);  // open price
   }else{
      request.price=SymbolInfoDouble(Symbol(),SYMBOL_BID);  // open price
   }
   //--- send a trade request
   MqlTradeResult result={};
      
   if(!OrderSend(request,result)){
      Print("OrderSend error %d",GetLastError());     // if unable to send the request, output the error code
      return false;
   }

   return true;
}

bool modifyPosition(ulong resultOrder,double newStopLoss,double newTakeProfit){
   if(!Trade.PositionModify(resultOrder, newStopLoss, newTakeProfit)){
      Print("Order Modification failed %d",GetLastError());     // if unable to send the request, output the error code
      return false;
   }
   
   return true;
}

bool isNewBar()
  {
//--- memorize the time of opening of the last bar in the static variable
   static datetime last_time=0;
//--- current time
   datetime lastbar_time=(datetime)SeriesInfoInteger(Symbol(),Period(),SERIES_LASTBAR_DATE);

//--- if it is the first call of the function
   if(last_time==0)
     {
      //--- set the time and exit
      last_time=lastbar_time;
      return(false);
     }

//--- if the time differs
   if(last_time!=lastbar_time)
     {
      //--- memorize the time and return true
      last_time=lastbar_time;
      return(true);
     }
//--- if we passed to this line, then the bar is not new; return false
   return(false);
  }
//+------------------------------------------------------------------+